
-- Other than using TweakIt's saved vars
-- an Initializing call to ExtendedInfoBar_Initialize from TweakIt.lua
-- And a call to ExtendedInfoBar_ToggleExtendedInfo from TweakIt_Settings (settings menu)
-- All code for this feature for this feature is self contained in this file.
local ExtendedInfoBar = {}
local ADDON_FEATURE_NAME = "ExtendedInfoBar"

local function BackpackAltFreeSlotType()
	local invManager = PLAYER_INVENTORY
	
	if invManager:IsBanking() then 
		return INVENTORY_BANK 
	elseif invManager:IsGuildBanking() then
		return INVENTORY_GUILD_BANK
	else
		return INVENTORY_BANK
	end
end

-- Only needs to be done when NOT banking. If Banking or Guild banking the
-- game will handle this itself.
local function OnRefreshMoney()
	local invManager = PLAYER_INVENTORY
	
	if not (invManager:IsBanking() or invManager:IsGuildBanking()) then
		ZO_CurrencyControl_SetSimpleCurrency(ZO_PlayerInventoryInfoBarAltMoney, CURT_MONEY, GetBankedCurrencyAmount(CURT_MONEY), ZO_KEYBOARD_BANKED_CURRENCY_OPTIONS)
	end
end
ZO_PreHook(ZO_InventoryManager, "RefreshMoney", OnRefreshMoney)

do
---[[
	local tBackpackLayouts = {
		[1] = BACKPACK_DEFAULT_LAYOUT_FRAGMENT.layoutData,
		[2] = BACKPACK_MENU_BAR_LAYOUT_FRAGMENT.layoutData,
		[3] = BACKPACK_BANK_LAYOUT_FRAGMENT.layoutData,
		[4] = BACKPACK_GUILD_BANK_LAYOUT_FRAGMENT.layoutData,
		[5] = BACKPACK_TRADING_HOUSE_LAYOUT_FRAGMENT.layoutData,
		[6] = BACKPACK_MAIL_LAYOUT_FRAGMENT.layoutData,
		[7] = BACKPACK_PLAYER_TRADE_LAYOUT_FRAGMENT.layoutData,
		[8] = BACKPACK_STORE_LAYOUT_FRAGMENT.layoutData,
		[9] = BACKPACK_FENCE_LAYOUT_FRAGMENT.layoutData,
		[10] = BACKPACK_LAUNDER_LAYOUT_FRAGMENT.layoutData,
	}
	for k,layoutData in pairs(tBackpackLayouts) do
		layoutData.bankInfoBarVisibilityOption = ZO_BACKPACK_LAYOUT_SHOW_ALL_BANK_INFO_BARS
	end
--]]
	PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].altFreeSlotType = BackpackAltFreeSlotType()
	--BACKPACK_BANK_LAYOUT_FRAGMENT.layoutData.bankInfoBarVisibilityOption = ZO_BACKPACK_LAYOUT_SHOW_ALL_BANK_INFO_BARS
end



local function OnRefreshTelvarStones()
	local invManager = PLAYER_INVENTORY
	
    if not invManager:IsBanking() then
		ZO_CurrencyControl_SetSimpleCurrency(ZO_PlayerInventoryInfoBarAltTelvarStones, CURT_TELVAR_STONES, GetBankedCurrencyAmount(CURT_TELVAR_STONES), ZO_KEYBOARD_BANKED_TELVAR_OPTIONS)
	end
		ZO_CurrencyControl_SetSimpleCurrency(ZO_PlayerInventoryInfoBarAltTelvarStones, CURT_TELVAR_STONES, GetBankedCurrencyAmount(CURT_TELVAR_STONES), ZO_KEYBOARD_BANKED_TELVAR_OPTIONS)
end
ZO_PreHook(ZO_InventoryManager, "RefreshTelvarStones", OnRefreshTelvarStones)

INVENTORY_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
	if(newState == SCENE_FRAGMENT_SHOWING) then
		PLAYER_INVENTORY:RefreshTelvarStones()
	end
end)


function ZO_SharedStoreManager:RefreshCurrency()
    self.currentMoney = GetCarriedCurrencyAmount(CURT_MONEY)
    self.currentAP = GetCarriedCurrencyAmount(CURT_ALLIANCE_POINTS)
    self.currentTelvarStones = GetCarriedCurrencyAmount(CURT_TELVAR_STONES)
	
	ZO_CurrencyControl_SetSimpleCurrency(ZO_PlayerInventoryInfoBarAltTelvarStones, CURT_TELVAR_STONES, GetBankedCurrencyAmount(CURT_TELVAR_STONES), ZO_KEYBOARD_BANKED_TELVAR_OPTIONS)
	
    ZO_CurrencyControl_SetSimpleCurrency(ZO_PlayerInventoryInfoBarTelvarStones, CURT_TELVAR_STONES, GetCarriedCurrencyAmount(CURT_TELVAR_STONES), ZO_KEYBOARD_CARRIED_TELVAR_OPTIONS)
end


--======================================================================--
--=====  Local: Independent   ==========================================--
--============ Functions in this section are independent ===============--
--== They do not make calls to any other function in this addons code ==--
--======================================================================--
-- Change bottom offset of Inventory to make room
-- for the larger info bar
local function ChangeFragmentLayouts()
	local tBackpackLayouts = {
	[1] = BACKPACK_BANK_LAYOUT_FRAGMENT.layoutData,
	[2] = BACKPACK_MENU_BAR_LAYOUT_FRAGMENT.layoutData,
	[3] = BACKPACK_MAIL_LAYOUT_FRAGMENT.layoutData,
	[4] = BACKPACK_PLAYER_TRADE_LAYOUT_FRAGMENT.layoutData,
	[5] = BACKPACK_STORE_LAYOUT_FRAGMENT.layoutData,
	[6] = BACKPACK_FENCE_LAYOUT_FRAGMENT.layoutData,
	[7] = BACKPACK_LAUNDER_LAYOUT_FRAGMENT.layoutData,
	[8] = BACKPACK_TRADING_HOUSE_LAYOUT_FRAGMENT.layoutData,
	}
	for k,v in pairs(tBackpackLayouts) do
		local adjustedOffset = v.inventoryBottomOffsetY - 30
		v.inventoryBottomOffsetY = adjustedOffset or -60
	end
	
	
	-- These don't have a layout_fragment, do it manually
	-- Their defaults are topOffsetY=45 bottomOffsetY= -30
	local function ChangeAnchors(control, topOffsetY, bottomOffsetY)
		control:ClearAnchors()
		control:SetAnchor(TOPLEFT, ZO_SharedRightPanelBackground, TOPLEFT, 0, topOffsetY)
		control:SetAnchor(BOTTOMLEFT, ZO_SharedRightPanelBackground, BOTTOMLEFT, 0, bottomOffsetY)
	end
	local controlsToReanchor = {
		ZO_PlayerBank, ZO_StoreWindow, ZO_BuyBack, ZO_RepairWindow,ZO_EnchantingTopLevelInventory, ZO_AlchemyTopLevelInventory,ZO_SmithingTopLevelCreationPanel, ZO_SmithingTopLevelRefinementPanelInventory, ZO_SmithingTopLevelResearchPanel, ZO_SmithingTopLevelDeconstructionPanelInventory, ZO_SmithingTopLevelImprovementPanel,
	}
	for k, control in pairs(controlsToReanchor) do
		ChangeAnchors(control, 45, -60)
	end
	-- ZO_SmithingTopLevelImprovementPanel Moves the entire panel up
	-- ZO_SmithingTopLevelImprovementPanelInventory moves up the bottom of the inv win
	-- to make room for the booster window -- Original anchors 45 & -175
	ChangeAnchors(ZO_SmithingTopLevelImprovementPanelInventory, 45, -205)
	
	-- Have to shrink the row controls for the Trait list, so I created my own
	-- rowTemplate resized it, & now recreate the slotPool
    SMITHING.researchPanel.slotPool = ZO_ControlPool:New("ExtendedInfoBar_SmithingResearchSlot", SMITHING.researchPanel.control)
end


-- ONLY the Smithing Improvement panel has an offsetY other than 0
-- But we must handle it for it to anchor correctly
local function ReanchorInfoBar(anchorControl, offsetX, offsetY)
	local offsetX = offsetX or 0 -- not used, always 0
	local offsetY = offsetY or 0
	
	-- Switch Parent so it shows/hides on its own
	ExtendedInfoBar.infoBar:SetParent(anchorControl)
	
	-- Set new anchor
	ExtendedInfoBar.infoBar:ClearAnchors()
	ExtendedInfoBar.infoBar:SetAnchor(TOPLEFT, anchorControl, BOTTOMLEFT, offsetX, offsetY)
	ExtendedInfoBar.infoBar:SetAnchor(TOPRIGHT, anchorControl, BOTTOMRIGHT, offsetX, offsetY)
end

-- Special code for FilterIt compatibility
-- to show the number of slots shown at the end of the freeSlots string
local function FormatFreeSlotsForFilterIt(inventoryType, freeSlotsString)
	if not TweakIt.addonsLoaded["FilterIt"] then return freeSlotsString end

	local function IsInvFragmentShowing(inventoryType)
		if inventoryType == INVENTORY_BACKPACK then
			local state = INVENTORY_FRAGMENT:GetState()
			return state == SCENE_FRAGMENT_SHOWING or state == SCENE_FRAGMENT_SHOWN
		end
		local state = BANK_FRAGMENT:GetState()
		return state == SCENE_FRAGMENT_SHOWING or state == SCENE_FRAGMENT_SHOWN
	end
	
	local function GetNumSlotsShown(inventoryType)
		local inventory = PLAYER_INVENTORY.inventories[inventoryType]
		local colorDrkOrange 	= "|cFFA500"	-- Dark Orange
		
		local numItems = 0
		if PLAYER_INVENTORY.inventories[inventoryType].listView and PLAYER_INVENTORY.inventories[inventoryType].listView.data then
			numItems = #PLAYER_INVENTORY.inventories[inventoryType].listView.data
		end
		return zo_strformat(colorDrkOrange.."(<<1>>)", numItems)
	end
	
	if IsInvFragmentShowing(inventoryType) then
		local numSlotsShown = GetNumSlotsShown(inventoryType)
		freeSlotsString = zo_strformat("<<1>> <<2>>", freeSlotsString, numSlotsShown)
	end
	return freeSlotsString
end

--================================================--
--=======  Update Free Slots PreHook: ============--
--=== ZO_InventoryManager:UpdateFreeSlots(..)  ===--
--================================================--
local function UpdateExtendedInfoFreeSlots(self, inventoryType)
	local updateInv, label, SI_FORMAT
	
	-- Get the label to be updated
	if inventoryType == INVENTORY_BACKPACK then
		label = ExtendedInfoBar.backpackFreeSlots
	elseif inventoryType == INVENTORY_BANK then
		label = ExtendedInfoBar.bankFreeSlots
	end
	
	if not label then return end
	
	-- Used to get the correct SI format
	local function GetSiFormat(inventoryType, isFull)
		if inventoryType == INVENTORY_BACKPACK then
			if isFull then return SI_INVENTORY_BACKPACK_COMPLETELY_FULL end
			return SI_INVENTORY_BACKPACK_REMAINING_SPACES
		end
		-- else must be the bank
		if isFull then return SI_INVENTORY_BANK_COMPLETELY_FULL end
		return SI_INVENTORY_BANK_REMAINING_SPACES
	end
	
	-- Get Info needed to format the slots used string
	local numSlotsShown -- nillable
	local numUsedSlots, numSlots = PLAYER_INVENTORY:GetNumSlots(inventoryType)
	local isInvFull = numUsedSlots >= numSlots
	local SI_FORMAT = GetSiFormat(inventoryType, isInvFull)
	local freeSlotsString = zo_strformat(SI_FORMAT, numUsedSlots, numSlots)
	
	freeSlotsString = FormatFreeSlotsForFilterIt(inventoryType, freeSlotsString)

	--local labelText = FormatText(SI_FORMAT, numUsedSlots, numSlots, numSlotsShown)
	label:SetText(freeSlotsString)
end

--======================================--
--=====  Show/Hide/Move Labels =========--
--======================================--
-- shows/hides the extended inventory info bar
local function HideExtendedInvInfo(shouldHide)
	ExtendedInfoBar.infoBar:SetHidden(shouldHide)
end

local function HideDefaultInfoBars(shouldHide)
	-- Backpack/Bank
	ZO_PlayerInventoryInfoBar:SetHidden(shouldHide)
	ZO_PlayerBankInfoBar:SetHidden(shouldHide)
	-- Store:
	ZO_StoreWindowInfoBar:SetHidden(shouldHide)
	ZO_BuyBackInfoBar:SetHidden(shouldHide)
	ZO_RepairWindowInfoBar:SetHidden(shouldHide)
	-- Crafting: Enchanting/Alchemy
	ZO_EnchantingTopLevelInventoryInfoBar:SetHidden(shouldHide)
	ZO_AlchemyTopLevelInventoryInfoBar:SetHidden(shouldHide)
	-- Crafting: Smithing
	ZO_SmithingTopLevelCreationPanelInfoBar:SetHidden(shouldHide)
	ZO_SmithingTopLevelRefinementPanelInventoryInfoBar:SetHidden(shouldHide)
	ZO_SmithingTopLevelDeconstructionPanelInventoryInfoBar:SetHidden(shouldHide)
	ZO_SmithingTopLevelImprovementPanelInventoryInfoBar:SetHidden(shouldHide)
	ZO_SmithingTopLevelResearchPanelInfoBar:SetHidden(shouldHide)
end

-- my eyes are bleeding from staring at all of these layouts




--======================================--
--====  Create/Initialize Labels  ======--
--======================================--
-- Called only on load to initialize current values
local function UpdateAllExtendedInfo()
    ZO_CurrencyControl_SetSimpleCurrency(ExtendedInfoBar.carriedMoney, CURT_MONEY, GetCurrentMoney(), INVENTORY_CURRENCY_OPTIONS)
	 
	ZO_CurrencyControl_SetSimpleCurrency(ExtendedInfoBar.bankedMoney, CURT_MONEY, GetBankedMoney(), INVENTORY_BANKED_CURRENCY_OPTIONS)
	
    ZO_CurrencyControl_SetSimpleCurrency(ExtendedInfoBar.alliancePoints, CURRENCY_TYPE_ALLIANCE_POINTS, GetAlliancePoints(), INVENTORY_ALLIANCE_POINTS_OPTIONS)
	
	-- Update Free Space Labels
	UpdateExtendedInfoFreeSlots(nil, INVENTORY_BACKPACK)
	UpdateExtendedInfoFreeSlots(nil, INVENTORY_BANK)
	
end

-- Fence transaction label update PreHook
local function OnUpdateTransactionLabel(self, totalTransactions, usedTransactions, transactionsRemainingString, transactionsFullString)
    local transactionString = (usedTransactions >= totalTransactions) and transactionsFullString or transactionsRemainingString
    ExtendedInfoBar.transactions:SetText(zo_strformat(transactionString, usedTransactions, totalTransactions))
end
-- FenceEnterSell PreHook - To show transaction label
local function OnFenceEnterSell(totalSells, sellsUsed)
    ExtendedInfoBar.transactions:SetHidden(false)
end
-- FenceEnterLaunder PreHook - To show transaction label
local function OnFenceEnterLaunder(totalLaunders, laundersUsed)
    ExtendedInfoBar.transactions:SetHidden(false)
end
-- FenceClosed PreHook - To hide transaction label
local function OnFenceClosed()
    ExtendedInfoBar.transactions:SetHidden(true)
end

local function InitializeExtendedInfo()
	local infoBar = CreateControlFromVirtual("ExtendedInfoBar", ZO_PlayerInventory, "ExtendedInfoBar")
	ExtendedInfoBar.infoBar = infoBar
	
	ExtendedInfoBar.backpackFreeSlots 	= infoBar:GetNamedChild("BackpackFreeSlots")
	ExtendedInfoBar.bankFreeSlots 		= infoBar:GetNamedChild("BankFreeSlots")
	
	ExtendedInfoBar.alliancePoints 	= infoBar:GetNamedChild("AlliancePoints")
	ExtendedInfoBar.carriedMoney 	= infoBar:GetNamedChild("CarriedMoney")
	ExtendedInfoBar.bankedMoney 	= infoBar:GetNamedChild("BankedMoney")
	
	local bankMoneyLabel	= infoBar:GetNamedChild("BankedMoneyLabel")
	bankMoneyLabel:SetText(GetString(SI_CURRENCY_YOUR_BANKED_GOLD))
	
	ExtendedInfoBar.transactions 	= infoBar:GetNamedChild("Transactions")
end

-- I really should have made this a seperate addon
-- I didn't think it would be this much work

local function InitializeCallbackHooks()
	-- For show/hide & updating the transaction label
    FENCE_MANAGER:RegisterCallback("FenceClosed", OnFenceClosed)
    FENCE_MANAGER:RegisterCallback("FenceEnterSell", OnFenceEnterSell)
    FENCE_MANAGER:RegisterCallback("FenceEnterLaunder", OnFenceEnterLaunder)
	ZO_PreHook(ZO_Fence_Keyboard, "UpdateTransactionLabel", OnUpdateTransactionLabel)
	
	--local inventoryScene = SCENE_MANAGER:GetScene("inventory")
	--inventoryScene:RegisterCallback("StateChange", function(oldState, newState)
	-- Unlike the game were only using a single infoBar & just re-anchoring it
	-- when it needs to be shown.
	local function stateChange(oldState, newState, anchorControl)
		if newState == SCENE_SHOWING then
			ReanchorInfoBar(anchorControl)
		end
	end
	
	-- As-If this addon hadn't already registered for enough stuff.....
	
	-- Register StateChange for all necessary fragments
    INVENTORY_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_PlayerInventory)
		-- If FilterIt is running, must update both labels
		-- to show the extra number of slots Shown info
		if TweakIt.addonsLoaded["FilterIt"] then
			UpdateExtendedInfoFreeSlots(nil, INVENTORY_BACKPACK)
			UpdateExtendedInfoFreeSlots(nil, INVENTORY_BANK)
		end
	end)
	
	BANK_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_PlayerBank) 
		-- If FilterIt is running, must update both labels
		-- to show the extra number of slots Shown info
		if TweakIt.addonsLoaded["FilterIt"] then
			UpdateExtendedInfoFreeSlots(nil, INVENTORY_BACKPACK)
			UpdateExtendedInfoFreeSlots(nil, INVENTORY_BANK)
		end
	end)
		
    STORE_FRAGMENT:RegisterCallback("StateChange",   function(oldState, newState)
		stateChange(oldState, newState, ZO_StoreWindow) end)
		
	BUY_BACK_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_BuyBack) end)
		
	REPAIR_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_RepairWindow) end)
		
	ENCHANTING_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_EnchantingTopLevelInventory) end)
		
	ALCHEMY_FRAGMENT:RegisterCallback("StateChange", function(oldState, newState)
		stateChange(oldState, newState, ZO_AlchemyTopLevelInventory) end)
	
	
	-- Smithing modes do not have their own fragments
	-- So we much PreHook SMITHING:SetMode(..)
	local function GetAnchorFromSmithingMode(mode)
		local anchorControl, offsetY
		
		if mode == SMITHING_MODE_CREATION then
			anchorControl = ZO_SmithingTopLevelCreationPanel
		elseif mode == SMITHING_MODE_REFINMENT then
			anchorControl = ZO_SmithingTopLevelRefinementPanelInventory
		elseif mode == SMITHING_MODE_DECONSTRUCTION then
			anchorControl = ZO_SmithingTopLevelDeconstructionPanelInventory
		elseif mode == SMITHING_MODE_IMPROVEMENT then
			anchorControl = ZO_SmithingTopLevelImprovementPanelInventoryBackpack
			offsetY = 145
		elseif mode == SMITHING_MODE_RESEARCH then
			anchorControl = ZO_SmithingTopLevelResearchPanel
		end
		return anchorControl, offsetY
	end
	
	-- So prehook SMITHING:SetMode(..) & anchor them
	local function OnSetMode(self, mode)
		local anchorControl, offsetY = GetAnchorFromSmithingMode(mode)
		ReanchorInfoBar(anchorControl, 0, offsetY)
	end
	ZO_PreHook(ZO_Smithing, "SetMode", OnSetMode)
end




--=====================================================--
--============= events & hooks ==============--
--=====================================================--

--======================================--
--=========  Events  ==========--
--======================================--
-- Update banked money amount
local function OnBankMoneyUpdated(eventCode, newBankedMoney, oldBankedMoney)
	ZO_CurrencyControl_SetSimpleCurrency(ExtendedInfoBar.bankedMoney, CURT_MONEY, newBankedMoney, INVENTORY_BANKED_CURRENCY_OPTIONS)
end

-- Update carried money amount
local function OnInvMoneyUpdated(eventCode, newMoney, oldMoney, reason) 
    ZO_CurrencyControl_SetSimpleCurrency(ExtendedInfoBar.carriedMoney, CURT_MONEY, newMoney, INVENTORY_CURRENCY_OPTIONS)
end


--===============================================--
--==== Global ====--
--===============================================--
-- Used to toggle the extended info bar visibility
-- and shows/hides the default bars
function ExtendedInfoBar_ToggleExtendedInfo(shouldHide)
	HideExtendedInvInfo(shouldHide)
	HideDefaultInfoBars(not shouldHide)
end

function ExtendedInfoBar_Initialize()
	local shouldHide = TweakIt.SavedVariables["HIDE_EXTENDED_INV_INFO"]
	if shouldHide then return end
	
	-- Create Extended Info Labels
	InitializeExtendedInfo()

	-- Register Callbacks & hooks for extended info anchors
	InitializeCallbackHooks()
	
	-- Set Initial values for infoBar labels
	UpdateAllExtendedInfo()
	
	-- Initialize visibility
	ExtendedInfoBar_ToggleExtendedInfo(shouldHide)
	
	-- I should really put this ^^ up there & reset the
	-- default layout if they turn it off...maybe later
	ChangeFragmentLayouts()
	
	-- Done here so we don't fire off any updates before were initialized
	ZO_PreHook(ZO_InventoryManager, "UpdateFreeSlots", UpdateExtendedInfoFreeSlots)
	
	EVENT_MANAGER:RegisterForEvent(ADDON_FEATURE_NAME, EVENT_BANKED_MONEY_UPDATE, OnBankMoneyUpdated)
	EVENT_MANAGER:RegisterForEvent(ADDON_FEATURE_NAME, EVENT_MONEY_UPDATE, OnInvMoneyUpdated)
end