
local LIBLA = LibStub:GetLibrary("LibLoadedAddons")

local ADDON_NAME	= "KeybindSearch"
local CODE_VERSION	= 1.0

local lastIndex = 1

local function DoSearch(_cEditBox, _bNextSearch)
	if not _cEditBox then return end
	
	local label = _cEditBox:GetParent():GetNamedChild("Label")
	label:SetHidden(true)
	
	_cEditBox:LoseFocus()
	
	local sSearchText = string.lower(_cEditBox:GetText())
	if not sSearchText or sSearchText == "" then return end
	local yOffsetPadding = 15
	
	local listData = ZO_KeybindingsList.data
	
	if not _bNextSearch then
		lastIndex = 1
	end
	
	for i = lastIndex+1, #listData do
		local layerName = string.lower(listData[i].data.layerName)
		local categoryName = string.lower(listData[i].data.categoryName)
		local localizedActionName = string.lower(listData[i].data.localizedActionName)
		
		if layerName and layerName:find(sSearchText) then
			ZO_KeybindingsListScrollBar:SetValue(listData[i].top-yOffsetPadding)
			lastIndex = i
			return
		elseif categoryName and categoryName:find(sSearchText) then
			ZO_KeybindingsListScrollBar:SetValue(listData[i].top-yOffsetPadding)
			lastIndex = i
			return
		elseif localizedActionName and localizedActionName:find(sSearchText) then
			ZO_KeybindingsListScrollBar:SetValue(listData[i].top-yOffsetPadding)
			lastIndex = i
			return
		end
	end
	local label = _cEditBox:GetParent():GetNamedChild("Label")
	label:SetHidden(false)
	lastIndex = 1
end

local function FindNext(_cNextBtn)
	local editBox = _cNextBtn:GetParent():GetNamedChild("EditBox")
	DoSearch(editBox, true)
end


local function SetUpKeybindingsSearch()
	if TweakIt.searchBox then return end
	
	-- Registers addon to loadedAddon library
	LIBLA:RegisterAddon(ADDON_NAME, CODE_VERSION)
	
	local searchBoxName = ZO_Keybindings:GetName() .. "TweakItSearchBox"
	
	local cSearchBox = WINDOW_MANAGER:CreateControl(searchBoxName, ZO_Keybindings, CT_EDITBOX)
	cSearchBox:ClearAnchors()
	cSearchBox:SetAnchor(TOPLEFT, ZO_Keybindings, TOPLEFT, 25, 25)
	cSearchBox:SetDimensions(200, 25)
	cSearchBox:SetMouseEnabled(true)
	TweakIt.searchBox = cSearchBox
	
	--local cNextBtn = WINDOW_MANAGER:CreateControlFromVirtual(cSearchBox:GetName().."NextBtn", cSearchBox, "ZO_DefaultTextButton")
	local cNextBtn = WINDOW_MANAGER:CreateControlFromVirtual(cSearchBox:GetName().."NextBtn", cSearchBox,  "ZO_BladeHeader")
	cNextBtn:ClearAnchors()
	cNextBtn:SetAnchor(LEFT, cSearchBox, RIGHT, 25, 0)
	cNextBtn:SetDimensions(60, 30)
	cNextBtn:SetText("Next")
	cNextBtn:SetHandler("OnClicked", FindNext)
	cNextBtn:SetClickSound("Click")
	
	cNextBtn.data = { tooltipText = "Find the next text match. " }
    cNextBtn:SetHandler("OnMouseEnter", ZO_Options_OnMouseEnter)
    cNextBtn:SetHandler("OnMouseExit", ZO_Options_OnMouseExit)
	--[[
	local cNextBtn = WINDOW_MANAGER:CreateControlFromVirtual(cNextBtn:GetName().."Bg", cNextBtn, "ZO_DefaultBackdrop")
	cNextBtn:ClearAnchors()
	cNextBtn:SetAnchorFill()
	--]]
	
	cSearchBox.bg = WINDOW_MANAGER:CreateControlFromVirtual(cSearchBox:GetName().."background", cSearchBox, "ZO_EditBackdrop")
	cSearchBox.bg:ClearAnchors()
	cSearchBox.bg:SetAnchorFill()
	cSearchBox.bg:SetDrawTier(1)
	
	cSearchBox.editbox = WINDOW_MANAGER:CreateControlFromVirtual(cSearchBox:GetName().."EditBox", cSearchBox, "ZO_DefaultEditForBackdrop")
	cSearchBox.editbox:ClearAnchors()
	cSearchBox.editbox:SetAnchorFill()
	cSearchBox.editbox:SetText("Enter Search Text")
	cSearchBox.editbox:SetHandler("OnEnter", DoSearch)
	cSearchBox.editbox:SetHandler("OnMouseDoubleClick", function() cSearchBox.editbox:Clear() end)
	cSearchBox.editbox.data = { tooltipText = "Enter text to search for and hit Enter. \nDouble click to clear text." }
    cSearchBox.editbox:SetHandler("OnMouseEnter", ZO_Options_OnMouseEnter)
    cSearchBox.editbox:SetHandler("OnMouseExit", ZO_Options_OnMouseExit)
	
	cSearchBox.label = WINDOW_MANAGER:CreateControl(cSearchBox:GetName() .. "Label", cSearchBox, CT_LABEL)
	cSearchBox.label:SetDimensions(200, 30)
	cSearchBox.label:ClearAnchors()
	cSearchBox.label:SetAnchor(TOPLEFT, cSearchBox, BOTTOMLEFT, 0, 5)
	cSearchBox.label:SetText("Not Found")
	cSearchBox.label:SetFont("ZoFontWinH4")
	cSearchBox.label:SetColor(1, 0, 0)
	cSearchBox.label:SetHidden(true)
end


function TweakIt.SetUpKeybindingsSearch()
	-- Initialize Keybindings Search Box
	SetUpKeybindingsSearch()
end