

------------------------------------------------------------------
--  LibFilterIt support --
-- I don't need the actualy libFilterIt library, but I need some 
-- of the constants for the filter types
------------------------------------------------------------------
local FILTERIT_VENDOR = 7


------------------------------------------------------------------
--  Initialize Strings  --
------------------------------------------------------------------
ZO_CreateStringId("TWEAKIT_EQUIPPED",  "Repair Equipped")
ZO_CreateStringId("TWEAKIT_ALL_EQUIPPED_ITEMS",  "Repair All Equipped Items?")
ZO_CreateStringId("TWEAKIT_EQUIPPED_WITH_GOLD",  "Repair Equipped (<<X:1>>|t24:24:EsoUI/Art/currency/currency_gold.dds|t)")
ZO_CreateStringId("TWEAKIT_SELLALL_JUNK_GOLD",  GetString(SI_SELL_ALL_JUNK_KEYBIND_TEXT).. " (<<X:1>>|t24:24:EsoUI/Art/currency/currency_gold.dds|t)")


-- Gets value for all junk items
local function GetJunkItemsValue()
	local tSlots = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots
	local iTotalValue = 0
	
	for k, tSlot in pairs(tSlots) do
		local bagId = tSlot.bagId
		local slotId = tSlot.slotIndex
		if IsItemJunk(bagId, slotId) and not IsItemStolen(bagId, slotId) then
			if not tSlot.FilterIt_CurrentFilter or tSlot.FilterIt_CurrentFilter == FILTERIT_VENDOR then
				local _, iStackSize, iSellPrice = GetItemInfo(bagId, slotId)
				iTotalValue = iTotalValue + (iSellPrice*iStackSize)
			end
		end
	end
	return iTotalValue
end

-- Get Repair cost for equipped items
local function GetEquippedItemsRepairCost()
	local tEquipSlots =
	{
		[1]		= EQUIP_SLOT_HEAD,
		[2]		= EQUIP_SLOT_NECK,
		[3]		= EQUIP_SLOT_CHEST,
		[4]		= EQUIP_SLOT_SHOULDERS,
		[5]		= EQUIP_SLOT_MAIN_HAND,
		[6]		= EQUIP_SLOT_OFF_HAND,
		[7]		= EQUIP_SLOT_WAIST,
		[8]		= EQUIP_SLOT_LEGS,
		[9]		= EQUIP_SLOT_FEET,
		[10]	= EQUIP_SLOT_COSTUME,
		[11]	= EQUIP_SLOT_RING1,
		[12]	= EQUIP_SLOT_RING2,
		[13]	= EQUIP_SLOT_HAND,
		[14]	= EQUIP_SLOT_BACKUP_MAIN,
		[15] 	= EQUIP_SLOT_BACKUP_OFF,
	}
	local iRepairCost = 0

	for k,v in pairs(tEquipSlots) do
		iRepairCost = iRepairCost + GetItemRepairCost(BAG_WORN, v)
	end
	return iRepairCost
end


-- Repair equipped items
local function RepairEquippedItems()
	local tEquipSlots =
	{
		[1]		= EQUIP_SLOT_HEAD,
		[2]		= EQUIP_SLOT_NECK,
		[3]		= EQUIP_SLOT_CHEST,
		[4]		= EQUIP_SLOT_SHOULDERS,
		[5]		= EQUIP_SLOT_MAIN_HAND,
		[6]		= EQUIP_SLOT_OFF_HAND,
		[7]		= EQUIP_SLOT_WAIST,
		[8]		= EQUIP_SLOT_LEGS,
		[9]		= EQUIP_SLOT_FEET,
		[10]	= EQUIP_SLOT_COSTUME,
		[11]	= EQUIP_SLOT_RING1,
		[12]	= EQUIP_SLOT_RING2,
		[13]	= EQUIP_SLOT_HAND,
		[14]	= EQUIP_SLOT_BACKUP_MAIN,
		[15] 	= EQUIP_SLOT_BACKUP_OFF,
	}

	for k,v in pairs(tEquipSlots) do
		if GetItemRepairCost(BAG_WORN, v) > 0 then
			RepairItem(BAG_WORN, v)
		end
	end
end



------------------------------------------------------------------
--  Change Store Keybind Strip Descriptor  --
------------------------------------------------------------------
local function GetRepairEquippedDescriptor()
	if TweakIt.SavedVariables["REPAIR_EQUIPPED_KEYBIND_BUTTON"] then
			-- Repair Equipped Items
		return {
				name = function() 
					local cost = GetEquippedItemsRepairCost()
					if GetCurrentMoney() >= cost then
						return zo_strformat(TWEAKIT_EQUIPPED_WITH_GOLD, ZO_CurrencyControl_FormatCurrency(cost))
					end
					return zo_strformat(TWEAKIT_EQUIPPED_WITH_GOLD, ZO_ERROR_COLOR:Colorize(ZO_CurrencyControl_FormatCurrency(cost)))
				end,
				keybind = "UI_SHORTCUT_SECONDARY",
				visible = function() return GetEquippedItemsRepairCost() > 0 end,
				callback = function()
					ZO_Dialogs_ShowDialog("TWEAKIT_EQUIPPED_CONFIRM", {cost = GetEquippedItemsRepairCost()})
				end,
			}
	else
        -- Repair All
		return {
            name = function()
                local cost = GetRepairAllCost()
                if GetCurrentMoney() >= cost then
                    return zo_strformat(SI_REPAIR_ALL_KEYBIND_TEXT, ZO_CurrencyControl_FormatCurrency(cost))
                end
                return zo_strformat(SI_REPAIR_ALL_KEYBIND_TEXT, ZO_ERROR_COLOR:Colorize(ZO_CurrencyControl_FormatCurrency(cost)))
            end,
            keybind = "UI_SHORTCUT_SECONDARY",
            visible = function() return GetRepairAllCost() > 0 end,
            callback = function()
                ZO_Dialogs_ShowDialog("REPAIR_ALL", {cost = GetRepairAllCost()})
            end,
        }
	end
end

local function TemporaryVisCheckForSellAllJunk()
	if not HasAnyJunk(BAG_BACKPACK) then return false end
	
	local tSlots = PLAYER_INVENTORY.inventories[INVENTORY_BACKPACK].slots
	
	for k, tSlot in pairs(tSlots) do
		local bagId = tSlot.bagId
		local slotId = tSlot.slotIndex
		if IsItemJunk(bagId, slotId) and not IsItemStolen(bagId, slotId) then
			return true
		end
	end
	return false
end
function TweakIt.InitStoreKeybindStripDescriptors()
	STORE_WINDOW.keybindStripDescriptor =    
	{
		alignment = KEYBIND_STRIP_ALIGN_CENTER,
		GetRepairEquippedDescriptor(),

		-- Sell All Junk
		{
			name = function() 
						return zo_strformat(TWEAKIT_SELLALL_JUNK_GOLD, ZO_CurrencyControl_FormatCurrency(GetJunkItemsValue()))
						--return GetString(SI_SELL_ALL_JUNK_KEYBIND_TEXT).." ("..ZO_CurrencyControl_FormatCurrency(GetJunkItemsValue())..")" 
						end,
			keybind = "UI_SHORTCUT_NEGATIVE",

			visible =   function()
							---------------------------------------------------------------------
							-- Temporary change until they fix stolen items showing up in the
							-- Sell All Junk keybindstrip
							--return HasAnyJunk(BAG_BACKPACK)
							return TemporaryVisCheckForSellAllJunk()
							---------------------------------------------------------------------
						end,
			callback =  function()
							ZO_Dialogs_ShowDialog("SELL_ALL_JUNK")
						end,
		},
	}
end
------------------------------------------------------------------
--  Create Dialog  --
------------------------------------------------------------------
function TweakIt.InitRepairEquippedDialog()
	ESO_Dialogs["TWEAKIT_EQUIPPED_CONFIRM"] =
	{
		title =
		{
			text = TWEAKIT_EQUIPPED,
		},
		mainText = 
		{
			text = TWEAKIT_ALL_EQUIPPED_ITEMS,
		},
		buttons =
		{
			[1] =
			{
				text =      SI_DIALOG_ACCEPT,
				callback =  RepairEquippedItems,
			},
			[2] =
			{
				text =       SI_DIALOG_DECLINE,
			},
		},
		updateFn = function(dialog)
			local cost = dialog.data.cost
			if cost > GetCurrentMoney() then
				ZO_Dialogs_UpdateButtonState(dialog, 1, BSTATE_DISABLED)
				ZO_Dialogs_UpdateDialogMainText(dialog, { text = SI_REPAIR_ALL_CANNOT_AFFORD })
			else
				ZO_Dialogs_UpdateButtonState(dialog, 1, BSTATE_NORMAL)
				ZO_Dialogs_UpdateDialogMainText(dialog, { text = TWEAKIT_ALL_EQUIPPED_ITEMS })
			end
			ZO_Dialogs_UpdateButtonCost(dialog, 1, cost)
		end,
	}
end

